
#ifndef __HANDLE__
#define __HANDLE__

#define  WINVER      0x0501
#define _WIN32_WINNT 0x0501
#define _WIN32_IE    0x0500

#define UNICODE
#define _UNICODE

#include <windows.h>
//#include <tlhelp32.h>

#define NT_SUCCESS(x) ((x) >= 0)
#define STATUS_SUCCESS					((NTSTATUS)0x00000000L)
#define STATUS_INFO_LENGTH_MISMATCH		((NTSTATUS)0xC0000004L)
#define STATUS_BUFFER_OVERFLOW			((NTSTATUS)0x80000005L)


#define SystemHandleInformation         16
#define ObjectBasicInformation          0
#define ObjectNameInformation           1
#define ObjectTypeInformation           2

DWORD
WINAPI
GetModuleBaseNameW(
    HANDLE  hProcess,
    HMODULE hModule,
    LPWSTR  lpBaseName,
    DWORD   nSize
    );

typedef struct _UNICODE_STRING {
	USHORT  Length;
	USHORT  MaximumLength;
	PWSTR   Buffer;
} UNICODE_STRING, *PUNICODE_STRING;

typedef LONG NTSTATUS;

typedef NTSTATUS (NTAPI *_NtQuerySystemInformation)(
    ULONG  SystemInformationClass,
    PVOID  SystemInformation,
    ULONG  SystemInformationLength,
    PULONG ReturnLength
    );
    
typedef NTSTATUS (NTAPI *_NtDuplicateObject)(
    HANDLE      SourceProcessHandle,
    HANDLE      SourceHandle,
    HANDLE      TargetProcessHandle,
    PHANDLE     TargetHandle,
    ACCESS_MASK DesiredAccess,
    ULONG       Attributes,
    ULONG       Options
    );
    
typedef NTSTATUS (NTAPI *_NtQueryObject)(
    HANDLE ObjectHandle,
    ULONG  ObjectInformationClass,
    PVOID  ObjectInformation,
    ULONG  ObjectInformationLength,
    PULONG ReturnLength
    );
    
typedef struct _IO_STATUS_BLOCK
{
    union
    {
        NTSTATUS Status;
        PVOID    Pointer;
    };
    
    ULONG_PTR    Information;
} IO_STATUS_BLOCK, *PIO_STATUS_BLOCK;

typedef VOID (WINAPI * PIO_APC_ROUTINE)(
    PVOID,
    PIO_STATUS_BLOCK,
    DWORD
    );

typedef VOID *POBJECT;

typedef struct _SYSTEM_HANDLE
{
    ULONG       ProcessId;
    BYTE        ObjectTypeNumber;
    BYTE        Flags;
    USHORT      Handle;
    PVOID       Object;
    ACCESS_MASK GrantedAccess;
} SYSTEM_HANDLE, *PSYSTEM_HANDLE;

typedef struct _SYSTEM_HANDLE_INFORMATION
{
    ULONG         HandleCount;
    SYSTEM_HANDLE Handles[1];
} SYSTEM_HANDLE_INFORMATION, *PSYSTEM_HANDLE_INFORMATION;

typedef enum _POOL_TYPE
{
    NonPagedPool,
    PagedPool,
    NonPagedPoolMustSucceed,
    DontUseThisType,
    NonPagedPoolCacheAligned,
    PagedPoolCacheAligned,
    NonPagedPoolCacheAlignedMustS
} POOL_TYPE, *PPOOL_TYPE;

typedef struct _OBJECT_TYPE_INFORMATION
{
    UNICODE_STRING  Name;
    ULONG           TotalNumberOfObjects;
    ULONG           TotalNumberOfHandles;
    ULONG           TotalPagedPoolUsage;
    ULONG           TotalNonPagedPoolUsage;
    ULONG           TotalNamePoolUsage;
    ULONG           TotalHandleTableUsage;
    ULONG           HighWaterNumberOfObjects;
    ULONG           HighWaterNumberOfHandles;
    ULONG           HighWaterPagedPoolUsage;
    ULONG           HighWaterNonPagedPoolUsage;
    ULONG           HighWaterNamePoolUsage;
    ULONG           HighWaterHandleTableUsage;
    ULONG           InvalidAttributes;
    GENERIC_MAPPING GenericMapping;
    ULONG           ValidAccess;
    BOOLEAN         SecurityRequired;
    BOOLEAN         MaintainHandleCount;
    USHORT          MaintainTypeList;
    POOL_TYPE       PoolType;
    ULONG           PagedPoolUsage;
    ULONG           NonPagedPoolUsage;
} OBJECT_TYPE_INFORMATION, *POBJECT_TYPE_INFORMATION;

PVOID GetLibraryProcAddress(PSTR LibraryName, PSTR ProcName)
{
    return GetProcAddress(GetModuleHandleA(LibraryName), ProcName);
}

#endif
